@@ -6,11 +6,13 @@ use Template;
 use Template::Timer;
 use NEXT;
 
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 __PACKAGE__->mk_accessors('template');
 __PACKAGE__->mk_accessors('include_path');
 
+*paths = \&include_path;
+
 =head1 NAME
 
 Catalyst::View::TT - Template View Class
@@ -192,6 +194,10 @@ checking and the chance of a memory leak:
 
     @{ $c->view('TT')->include_path } = qw/path another_path/;
 
+If you are calling C<render> directly then you can specify dynamic paths by 
+having a C<additional_template_paths> key with a value of additonal directories
+to search. See L<CAPTURING TEMPLATE OUTPUT> for an example showing this.
+
 =head2 RENDERING VIEWS
 
 The view plugin renders the template specified in the C<template>
@@ -204,13 +210,12 @@ item in the stash.
     }
 
 If a class item isn't defined, then it instead uses the
-current match, as returned by C<$c-E<gt>match>.  In the above 
+current match, as returned by C<< $c->match >>.  In the above 
 example, this would be C<message>.
 
 The items defined in the stash are passed to the Template Toolkit for
 use as template variables.
 
-sub message : Global {
     sub default : Private {
         my ( $self, $c ) = @_;
         $c->stash->{template} = 'message.tt2';
@@ -233,11 +238,35 @@ These can be accessed from the template in the usual way:
     The name is [% name %]
 
 
-The output generated by the template is stored in
-C<$c-E<gt>response-E<gt>output>.
+The output generated by the template is stored in C<< $c->response->body >>.
+
+=head2 CAPTURING TEMPLATE OUTPUT
+
+If you wish to use the output of a template for some other purpose than
+displaying in the response, e.g. for sending an email, this is possible using
+L<Catalyst::Plugin::Email> and the L<render> method:
+
+  sub send_email : Local {
+    my ($self, $c) = @_;
+    
+    $c->email(
+      header => [
+        To      => 'me@localhost',
+        Subject => 'A TT Email',
+      ],
+      body => $c->view('TT')->render($c, 'email.tt', {
+        additional_template_paths => [ $c->config->{root} . '/email_templates'],
+        email_tmpl_param1 => 'foo'
+        }
+      ),
+    );
+  # Redirect or display a message
+  }
 
 =head2 TEMPLATE PROFILING
 
+See L<C<TIMER>> property of the L<config> method.
+
 =head2 METHODS
 
 =over 4
@@ -290,7 +319,7 @@ sub new {
     if ( $config->{TIMER} ) {
         if ( $config->{CONTEXT} ) {
             $c->log->error(
-                'Cannot use Template::Timer - a TT CONFIG is already defined'
+                'Cannot use Template::Timer - a TT CONTEXT is already defined'
             );
         }
         else {
@@ -307,9 +336,22 @@ sub new {
         if ( ref($config->{PROVIDERS}) eq 'ARRAY') {
             foreach my $p (@{$config->{PROVIDERS}}) {
                 my $pname = $p->{name};
-                eval "require Template::Provider::$pname";
+                my $prov = 'Template::Provider';
+                if($pname eq '_file_')
+                {
+                    $p->{args} = { %$config };
+                }
+                else
+                {
+                    $prov .="::$pname" if($pname ne '_file_');
+                }
+                eval "require $prov";
                 if(!$@) {
-                    push @providers, "Template::Provider::${pname}"->new($p->{args});
+                    push @providers, "$prov"->new($p->{args});
+                }
+                else
+                {
+                    $c->log->warn("Can't load $prov, ($@)");
                 }
             }
         }
@@ -320,33 +362,40 @@ sub new {
     }
 
     my $self = $class->NEXT::new(
-        $c,
-        {   template => Template->new($config) || do {
-                my $error = Template->error();
-                $c->log->error($error);
-                $c->error($error);
-                return undef;
-            },
-            %{$config},
-        },
+        $c, { %$config }, 
     );
+
+    # Set base include paths. Local'd in render if needed
     $self->include_path($config->{INCLUDE_PATH});
+    
     $self->config($config);
 
+    # Creation of template outside of call to new so that we can pass [ $self ]
+    # as INCLUDE_PATH config item, which then gets ->paths() called to get list
+    # of include paths to search for templates.
+   
+    # Use a weakend copy of self so we dont have loops preventing GC from working
+    my $copy = $self;
+    Scalar::Util::weaken($copy);
+    $config->{INCLUDE_PATH} = [ sub { $copy->paths } ];
+    
+    $self->{template} = 
+        Template->new($config) || do {
+            my $error = Template->error();
+            $c->log->error($error);
+            $c->error($error);
+            return undef;
+        };
+
+
     return $self;
 }
 
 =item process
 
-Renders the template specified in C<$c-E<gt>stash-E<gt>{template}> or
-C<$c-E<gt>action> (the private name of the matched action. 
-Template variables are set up from the contents of C<$c-E<gt>stash>, 
-augmented with C<base> set to C<$c-E<gt>req-E<gt>base>, C<c> to C<$c> and 
-C<name> to C<$c-E<gt>config-E<gt>{name}>. Alternately, the C<CATALYST_VAR>
-configuration item can be defined to specify the name of a template
-variable through which the context reference (C<$c>) can be accessed.
-In this case, the C<c>, C<base> and C<name> variables are omitted.
-Output is stored in C<$c-E<gt>response-E<gt>output>.
+Renders the template specified in C<< $c->stash->{template} >> or
+C<< $c->action >> (the private name of the matched action.  Calls L<render> to
+perform actual rendering. Output is stored in C<< $c->response->body >>.
 
 =cut
 
@@ -361,24 +410,14 @@ sub process {
         return 0;
     }
 
-    $c->log->debug(qq/Rendering template "$template"/) if $c->debug;
+    my $output = $self->render($c, $template);
 
-    my $output;
-    my $vars = { $self->template_vars($c) };
-
-    unshift @{ $self->include_path },
-      @{ $c->stash->{additional_template_paths} }
-      if ref $c->stash->{additional_template_paths};
-    unless ( $self->template->process( $template, $vars, \$output ) ) {
-        my $error = $self->template->error;
-        $error = qq/Couldn't render template "$error"/;
+    if (UNIVERSAL::isa($output, 'Template::Exception')) {
+        my $error = qq/Coldn't render template "$output"/;
         $c->log->error($error);
         $c->error($error);
         return 0;
     }
-    splice @{ $self->include_path }, 0,
-      scalar @{ $c->stash->{additional_template_paths} }
-      if ref $c->stash->{additional_template_paths};
 
     unless ( $c->response->content_type ) {
         $c->response->content_type('text/html; charset=utf-8');
@@ -389,9 +428,50 @@ sub process {
     return 1;
 }
 
+=item render($c, $template, \%args)
+
+Renders the given template and returns output, or a L<Template::Exception>
+object upon error. 
+
+The template variables are set to C<%$args> if $args is a hashref, or 
+$C<< $c->stash >> otherwise. In either case the variables are augmented with 
+C<base> set to C< << $c->req->base >>, C<c> to C<$c> and C<name> to
+C<< $c->config->{name} >>. Alternately, the C<CATALYST_VAR> configuration item
+can be defined to specify the name of a template variable through which the
+context reference (C<$c>) can be accessed. In this case, the C<c>, C<base> and
+C<name> variables are omitted.
+
+C<$template> can be anything that Template::process understands how to 
+process, including the name of a template file or a reference to a test string.
+See L<Template::process|Template/process> for a full list of supported formats.
+
+=cut
+
+sub render {
+    my ($self, $c, $template, $args) = @_;
+
+    $c->log->debug(qq/Rendering template "$template"/) if $c->debug;
+
+    my $output;
+    my $vars = { 
+        (ref $args eq 'HASH' ? %$args : %{ $c->stash() }),
+        $self->template_vars($c)
+    };
+
+    local $self->{include_path} = 
+        [ @{ $vars->{additional_template_paths} }, @{ $self->{include_path} } ]
+        if ref $vars->{additional_template_paths};
+
+    unless ($self->template->process( $template, $vars, \$output ) ) {
+        return $self->template->error;  
+    } else {
+        return $output;
+    }
+}
+
 =item template_vars
 
-Returns a list of keys/values to be used as the variables in the
+Returns a list of keys/values to be used as the catalyst variables in the
 template.
 
 =cut
@@ -407,10 +487,9 @@ sub template_vars {
         c    => $c,
         base => $c->req->base,
         name => $c->config->{name}
-      ),
-      %{ $c->stash() }
-
+      )
 }
+
 =item config
 
 This method allows your view subclass to pass additional settings to
